﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace FileSystem {
  public abstract class Entry { // the base name of the composite type

     protected string name;    //  the entry's name; accessible to the subclass code

     // get and set the value of name:
     public string Name {
       get { return name; }
       set { name = value; }
     }

     // returns a (long) string that lists names of all Entries within the current one.
     // Note that  "this"  refers to the current object being analyzed
     public override string ToString() { return this.ToString(""); }
     public abstract string ToString(string indent);


    // searches  path,  starting from this Entry,  to find and return the named Entry
    //   Param:  path  is the file path to search
    //   Returns: the Entry located at the end of  path
    public Entry searchE(string path) { 
        // convert  path  into a *list* of individual names and start the search:
        return searchE(new List<string>(path.Split('/')));   // the arg is a little ugly...
    }
    
   public Entry searchE(List<string> names) {
   if (names.Count == 0) { return this; }   // empty path? finished.
   else if (this is Folder) {
          Entry next = ((Folder)this).find(names[0]);  // look further...
          if (next == null) { return null; }   // bad path
          names.RemoveAt(0);    // finished with the front name on the list
          return next.searchE(names);   // finish search with remaining names
        }
   else if (this is Shortcut) {
          Entry next = ((Shortcut)this).Link;  // "jump" link
          if (next == null) { return null; }   //  oops -- a bad link
          return next.searchE(names);   // continue search from the jump
        }
   else if (this is TextFile) {
          return null;  }  // bad path
   else { return null; }    // this option should never be used...
   }

         
  }
}
