﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace FileSystem {
  // models a folder as a directory of Entries, indexed by their names:
  public class Folder : Entry {
    private Dictionary<string, Entry> directory;

    public Folder(string name) {
      base.name = name;  // save name in class Entry part of this object
      directory = new Dictionary<string, Entry>();  // initially, folder is empty
    }

    // returns the Entry with  name.  If name is bogus, returns null.
    public Entry find(string name) {
      Entry ans = null;
      if (directory.ContainsKey(name)) { ans = directory[name]; }
      return ans;
    }

    // adds a new Entry, e, to this folder
    public void add(Entry e) {
      if (!directory.ContainsKey(e.Name)) {
        directory[e.Name] = e;
      }
    }

    // returns a list of all the Entry objects saved in this folder
    public List<Entry> getListing() {
      List<Entry> ans = new List<Entry>();
      foreach (var pair in directory) { ans.Add(pair.Value); }
      return ans;
    }


    // searches  path,  starting from this Folder,  to find and return the named Entry
    // Param:  path - a string of form "Name1/Name2/.../Namem", e.g.,  "F/a.txt"
    // Returns: the Entry named by the path.  If path is invalid, returns  null
    // Example: say that  root  is handle to "Root" folder, which holds subfolder "F"
    //   which itself holds file  "a.txt".
    //   Then,  root.search("F/a.txt")  returns handle to TextFile object named "a.txt"
    public Entry search(string path)  {
      string[] names = path.Split('/');
      Entry currentEntry = this;    // currentEntry  remembers how far we searched
      foreach(string nam in names) {
          // loop invariant:  the path,  nam_0/.../nam_i-1,  led us to  currentEntry
          //   and we have  nam  to examine next:
          if (currentEntry is Folder) {  // is OK to go one level lower ?
              currentEntry = ((Folder)currentEntry).find(nam);   // yes, do it
          }
          else { currentEntry = null;  break; }   // bad path name, give up
          // "jump" the shortcut(s) in the path:
          while (currentEntry is Shortcut) { currentEntry = ((Shortcut)currentEntry).Link; }
      }
      // loop has ended --- the path names led us to here:
      return currentEntry;
    }


    // returns a (long) string that lists names of all Entries within the current one.
    public override string ToString() { return this.ToString(""); }
    public override string ToString(string indent) {
        string ans = indent + "Folder:  " + this.name + " containing:\r\n";
        List<Entry> contents = this.getListing(); // get all entries in folder
        foreach (Entry entry in contents) {
            ans = ans + entry.ToString(indent + "   ");
        }
        ans = ans + indent + "End Folder\r\n";
        return ans;
    }
 
  }
 }
