
"""Modelling a file system as nested dictionaries:

   FileSystem ::=  Folder
   
   Entry ::=  Textfile  |  Shortcut  |  Folder
   Folder ::=  { "name": String,  "files": {String: Entry} }
   TextFile ::=  { "name: String,  "lines": [String] }
   Shortcut ::=  { "name": String,  "link": Entry }
"""

### Functions for Entry:  
#     Entry ::=  Textfile  |  Shortcut  |  Folder

# functions that check the "type" of an Entry:  
def isFolder(e) : return "files" in e
def isTextFile(e) : return "lines" in e
def isShortcut(e) : return "link" in e

# toString  function, generates a pretty-printed string of Entry e
def toString(e, indent = "") :
    """params:  e - the Entry to print
                indent - leading blanks (optional)
       returns: formatted string of  e's  contents
    """
    if isFolder(e) :
       ans = indent + "Folder " + e["name"] + " containing:\n";
       for name in e["files"].keys() : 
            ans = ans + toString(e["files"][name], indent + "   ")
       ans = ans + indent + "End Folder\n";
       return ans; 
    elif isTextFile(e) :
        return indent + "TextFile " + e["name"] + "\n"   
    elif isShortcut(e) :
        return indent + "Shortcut " + e["name"] + " -> " + e["link"]["name"] + "\n";  
    else : return "toString error"
    

    
### Functions for Textfile:
#     TextFile ::=  { "name: String,  "lines": [String] }
def makefile(name):  return {"name": name, "lines":[] }
def addline(f, text):  f["lines"].append(text)   



### Functions for Shortcut:
#     Shortcut ::=  { "name": String,  "link": Entry }  
def makeshortcut(name, link): return {"name": name, "link": link}
  
 
 
### Functions for Folder:
#     Folder ::=  { "name": String,  "files": {String: Entry} }
def makefolder(name): return {"name": name, "files": {} }
def addentry(folder, e):  folder["files"][e["name"]] = e

# search function:
def search(e, path) :
    """params:  e - an Entry, should be a Folder
       path: a string of form "Name1/Name2/.../Namem", e.g.,  "F/G/a.txt"
       returns: the handle to the Entry named by  path
    """
    if isFolder(e) :
        names = path.split("/")  # make a list of names to search
        current = e
        for nam in names :
            while isShortcut(current):  current = current["link"]
            if isFolder(current): current = current["files"][nam]
            else :  break   # end of the path, for good or bad...    
        return current
    else : return "error"


  

### MAKE A FILE SYSTEM: 

filesystem = makefolder("Root")

a = makefile("a.txt")
addline(a, "I am file a.txt");  addline(a, "End of a.txt")
addentry(filesystem, a)

f = makefolder("F")
addentry(filesystem, f)

b = makefile("b.txt")
addline(a, "I am file b.txt.  The end.");  
addentry(f, b)

s = makeshortcut("LinkTo a", a)
addentry(f, s)

g = makefolder("G")
addentry(f, g)

c = makefile("c.txt")
addline(a, "I am file c.txt");  addline(a, "End")
addentry(g, c)

addentry(g, makeshortcut("LinkTo F", f))

h = makefolder("H")
addentry(filesystem, h)

### Play with it:

print "FILE SYSTEM CONTENTS:"
print toString(filesystem)

print "Search and print  F/G/c.txt:"
t = search(filesystem, "F/G/c.txt")
print toString(t), "  ", t

raw_input("done.")






